<?php

namespace Sale\Handlers\PaySystem;

use Bitrix\Main\Error;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\Request;
use Bitrix\Main\Text\Encoding;
use Bitrix\Main\Type\DateTime;
use Bitrix\Main\Web\HttpClient;
use Bitrix\Sale\BusinessValue;
use Bitrix\Sale\PaySystem;
use Bitrix\Sale\Payment;

Loc::loadMessages(__FILE__);

/**
 * Class CryptexyHandler
 * @package Sale\Handlers\PaySystem
 */
class CryptexyHandler extends PaySystem\ServiceHandler
{
    /**
     * @param Payment $payment
     * @param Request|null $request
     * @return PaySystem\ServiceResult
     */
    public function initiatePay(Payment $payment, Request $request = null)
    {
        $params = array(
            'PS_MODE' => $this->service->getField('PS_MODE'),
            'BX_PAYSYSTEM_CODE' => $this->service->getField('ID'),
            'ORDER_ID' => $payment->getOrderId()
        );

        $this->setExtraParams($params);

        return $this->showTemplate($payment, "template");
    }

    /**
     * @return array
     */
    public static function getIndicativeFields()
    {
        return array('verification_code','is_paid');
    }

    /**
     * @param Request $request
     * @return mixed
     */
    public function getPaymentIdFromRequest(Request $request)
    {
        $order=\Bitrix\Sale\Order::load($request->get('orderId'));
        foreach($order->getPaymentCollection() as $payment){
            $l[]=$payment->getField("ID");
        }
        // $l=$order->getPaymentSystemId();
        return current($l);
    }


    /**
     * @param Payment $payment
     * @param Request $request
     * @return bool
     */
    private function isCorrectSum(Payment $payment, Request $request)
    {
        $sum = $request->get('paid');
        $paymentSum = $this->getBusinessValue($payment, 'PAYMENT_SHOULD_PAY');

        return roundEx($paymentSum, 2) == roundEx($sum, 2);
    }

    /**
     * @param PaySystem\ServiceResult $result
     * @param Request $request
     * @return mixed
     */
    public function sendResponse(PaySystem\ServiceResult $result, Request $request)
    {
        global $APPLICATION;
        $APPLICATION->RestartBuffer();

        $data = $result->getData();

        header('Content-Type: text/html');
        header('Pragma: no-cache');

        if($data['code'] != 0)
            echo 'ok';
        else
            echo 'error';

        die();
    }

    /**
     * @param Payment $payment
     * @param Request $request
     * @return PaySystem\ServiceResult
     */
    public function processRequest(Payment $payment, Request $request)
    {

        $result = new PaySystem\ServiceResult();

        $verification_code = $request->get('verification_code');
        $is_paid = $request->get('is_paid');
        $code = $request->get('code');

        $data['verification_code'] = $verification_code;
        $data['is_paid'] = $is_paid;
        $data['code'] = $code;
        
        if($verification_code != "" && $is_paid == 1)
        {            
            $order=\Bitrix\Sale\Order::load($request->get('orderId'));
            foreach($order->getPaymentCollection() as $payment){
                $l[]=$payment->getField("ID");
            }

            $fields = array(
                "PS_STATUS_MESSAGE" => '',
                "PS_CURRENCY" => $payment->getField('CURRENCY'),
                "PS_RESPONSE_DATE" => new DateTime(),
                "PS_INVOICE_ID" => current($l),
                "PS_SUM" => $request->get('paid'),
            );       

            if ($this->isCorrectSum($payment, $request)){

                $data['CODE'] = $code;
                $fields["PS_STATUS"] = "Y";
                $fields["PS_STATUS_CODE"] = 1;
                $fields['PS_STATUS_DESCRIPTION'] = Loc::getMessage('SALE_HPS_CRYPTEXY_SUCCESS_STATUS');
                $result->setOperationType(PaySystem\ServiceResult::MONEY_COMING);
            }else{
                $sum = $request->get('paid');
                $paymentSum = $this->getBusinessValue($payment, 'PAYMENT_SHOULD_PAY');
                if($sum > $paymentSum){
                    $message =  Loc::getMessage('SALE_HPS_CRYPTEXY_PAYED_MORE');
                }elseif($sum < $paymentSum){
                    $message =  Loc::getMessage('SALE_HPS_CRYPTEXY_PAYED_LESS');
                }
                $data['CODE'] = 0;
                $fields["PS_STATUS"] = "N";
                $fields["PS_STATUS_CODE"] = 0;
                $fields['PS_STATUS_DESCRIPTION'] = $message;
                // $result->addError(new Error($message));
            }

            $result->setPsData($fields);
        }
        else
        {
            $data['CODE'] = 0;
            $result->addError(new Error(Loc::getMessage('SALE_HPS_CRYPTEXY_ERROR_STATUS')));
        }


        $result->setData($data);


        if (!$result->isSuccess())
        {
            PaySystem\ErrorLog::add(array(
                'ACTION' => "processRequest",
                'MESSAGE' => join('\n', $result->getErrorMessages())
            ));
        }

        return $result;
    }

    /**
     * @param Payment $payment
     * @return bool
     */
    protected function isTestMode(Payment $payment = null)
    {
        return ($this->getBusinessValue($payment, 'PS_IS_TEST') == 'Y');
    }

    /**
     * @return array
     */
    public function getCurrencyList()
    {
        return array('RUB');
    }


}